#include "Blog_MyAsset.h"
#include "MyAssetFactory.h"
#include "MyAsset.h"

UMyAssetFactory::UMyAssetFactory(const FObjectInitializer& ObjectInitializer)
	: Super(ObjectInitializer)
{
	SupportedClass = UMyAsset::StaticClass();
	bCreateNew = false;
	bEditorImport = true;
	bText = true;
	Formats.Add(TEXT("myasset;My Asset"));
}

bool UMyAssetFactory::DoesSupportClass(UClass* Class)
{
	return (Class == UMyAsset::StaticClass());
}

UClass* UMyAssetFactory::ResolveSupportedClass()
{
	return UMyAsset::StaticClass();
}

UObject* UMyAssetFactory::FactoryCreateNew(UClass* InClass, UObject* InParent, FName InName, EObjectFlags Flags, UObject* Context, FFeedbackContext* Warn)
{
	UMyAsset* NewMyAsset = CastChecked<UMyAsset>(StaticConstructObject(InClass, InParent, InName, Flags));
	return NewMyAsset;
}

UObject* UMyAssetFactory::FactoryCreateText(UClass* InClass, UObject* InParent, FName InName, EObjectFlags Flags, UObject* Context, const TCHAR* Type, const TCHAR*& Buffer, const TCHAR* BuferEnd, FFeedbackContext* Warn)
{
	TArray<FString> Values;
	FString(Buffer).ParseIntoArray(&Values, TEXT(","), true);

	UMyAsset* NewMyAsset = CastChecked<UMyAsset>(StaticConstructObject(InClass, InParent, InName, Flags));
	if(NewMyAsset && (3 <= Values.Num()))
	{
		NewMyAsset->ValueA = FCString::Atoi(*Values[0]);
		NewMyAsset->ValueB = FCString::Atoi(*Values[1]);
		NewMyAsset->ValueC = FCString::Atoi(*Values[2]);

		if(!NewMyAsset->AssetImportData)
		{
			NewMyAsset->AssetImportData = ConstructObject<UAssetImportData>(UAssetImportData::StaticClass(), NewMyAsset);
		}
		NewMyAsset->AssetImportData->SourceFilePath = FReimportManager::SanitizeImportFilename(CurrentFilename, NewMyAsset);
		NewMyAsset->AssetImportData->SourceFileTimestamp = IFileManager::Get().GetTimeStamp(*CurrentFilename).ToString();
		NewMyAsset->AssetImportData->bDirty = false;
	}

	return NewMyAsset;
}

bool UMyAssetFactory::CanReimport(UObject* Obj, TArray<FString>& OutFilenames)
{
	UMyAsset* MyAsset = Cast<UMyAsset>(Obj);
	if(MyAsset && MyAsset->AssetImportData)
	{
		OutFilenames.Add(FReimportManager::ResolveImportFilename(MyAsset->AssetImportData->SourceFilePath, MyAsset));
		return true;
	}
	return false;
}
void UMyAssetFactory::SetReimportPaths(UObject* Obj, const TArray<FString>& NewReimportPaths)
{
	UMyAsset* MyAsset = Cast<UMyAsset>(Obj);
	if(MyAsset && ensure(NewReimportPaths.Num() == 1))
	{
		MyAsset->AssetImportData->SourceFilePath = FReimportManager::ResolveImportFilename(NewReimportPaths[0], MyAsset);
	}
}
EReimportResult::Type UMyAssetFactory::Reimport(UObject* Obj)
{
	UMyAsset* MyAsset = Cast<UMyAsset>(Obj);
	if(!MyAsset)
	{
		return EReimportResult::Failed;
	}

	const FString Filename = FReimportManager::ResolveImportFilename(MyAsset->AssetImportData->SourceFilePath, MyAsset);
	if(!Filename.Len() || IFileManager::Get().FileSize(*Filename) == INDEX_NONE)
	{
		return EReimportResult::Failed;
	}

	EReimportResult::Type Result = EReimportResult::Failed;
	if(UFactory::StaticImportObject(MyAsset->GetClass(), MyAsset->GetOuter(), *MyAsset->GetName(), RF_Public | RF_Standalone, *Filename, NULL, this))
	{
		if (MyAsset->GetOuter())
		{
			MyAsset->GetOuter()->MarkPackageDirty();
		}
		else
		{
			MyAsset->MarkPackageDirty();
		}
		return EReimportResult::Succeeded;
	}

	return EReimportResult::Failed;
}